﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Collections.Generic;
using HeuristicLab.BioBoost.Representation;

namespace HeuristicLab.BioBoost.Operators.Transformation {

  [StorableClass]
  public class FeedstockInflater : FeedstockTransformer {

    #region Construction & Cloning
    [StorableConstructor]
    protected FeedstockInflater(bool isDeserializing) : base(isDeserializing) {}
    protected FeedstockInflater(FeedstockInflater orig, Cloner cloner) : base(orig, cloner) {}
    public FeedstockInflater() {}
    public override IDeepCloneable  Clone(Cloner cloner) {
      return new FeedstockInflater(this, cloner);
    }
    #endregion

    public override IOperation Apply() {
      var product = ProductName;
      var potentials = GetFromProblemData<DoubleArray>(LayerDescriptor.PotentialsFromProblemData.NameWithPrefix(product));
      var utilizations = GetFromScope<RealVector>(LayerDescriptor.Utilizations.NameWithPrefix(product));
      if (potentials != null && utilizations != null) {
        var newUtilizations = new List<double>();
        var skipped = 0;
        for (int i = 0; i < potentials.Length; i++) {
          if (potentials[i] > 0) {
            newUtilizations.Add(utilizations[i - skipped]);
          } else {
            skipped++;
            newUtilizations.Add(0);
          }
        }
        TransformValue(LayerDescriptor.Utilizations.NameWithPrefix(product), new RealVector(newUtilizations.ToArray()));
      }
      return base.Apply();
    }
  }
}
